@@ -46,19 +46,15 @@ extern "C"
 
 	typedef struct _CxpProfile CxpProfile;
 	typedef struct _CxpProfileClass CxpProfileClass;
-	typedef struct _CxpProfilePrivate CxpProfilePrivate;
 
 	struct _CxpProfile
 	{
 		GObject parent;
-		/* private */
-		CxpProfilePrivate *priv;
 	};
 
 	struct _CxpProfileClass
 	{
 		GObjectClass parent;
-		/* class members */
 	};
 
 /* used by CXP_TYPE_PROFILE */
