@@ -31,12 +31,14 @@ extern "C"
 #endif
 
 #include <gtk/gtk.h>
+#include <time.h>
 
 void add_pixmap_directory (const gchar * directory);
 GtkWidget *create_pixmap (GtkWidget * widget, const gchar * filename);
 GdkPixbuf *create_pixbuf (const gchar * filename);
 gchar *cxp_path_get_basename_of_utf8 (const gchar * file_name);
 gchar *cxp_path_get_dirname_of_utf8 (const gchar * file_name);
+gchar *cxp_strftime (const time_t * timep);
 
 #ifdef __cplusplus
 }
