@@ -36,11 +36,15 @@
  * @brief private member of CxpHandler
  */
 
-struct _CxpHandlerPrivate
+typedef struct
 {
 	CxpProfile *profile;
 	gboolean dispose_has_run;	/**< Is dispose funciton executed */
-};
+} CxpHandlerPrivate;
+
+#define CXP_HANDLER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CXP_TYPE_HANDLER, CxpHandlerPrivate))
+
+static GObjectClass *parent_class = NULL;
 
 /*
  * privete methods
@@ -84,14 +88,13 @@ GType cxp_handler_get_type (void)
  */
 static void cxp_handler_init (GTypeInstance * instance, gpointer g_class)
 {
+	CxpHandlerPrivate *priv = CXP_HANDLER_GET_PRIVATE(instance);
+
 	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
-	CxpHandler *self = CXP_HANDLER (instance);
-
-	self->priv = g_new (CxpHandlerPrivate, 1);
-	self->priv->profile = cxp_profile_new ("common", "handler");
-	self->priv->dispose_has_run = FALSE;
+	priv->profile = cxp_profile_new ("common", "handler");
+	priv->dispose_has_run = FALSE;
 }
 
 /**
@@ -104,26 +107,30 @@ static void cxp_handler_class_init (gpointer g_class, gpointer g_class_data)
 
 	gobject_class->dispose = cxp_handler_dispose;
 	gobject_class->finalize = cxp_handler_finalize;
+
+	g_type_class_add_private (g_class, sizeof (CxpHandlerPrivate));
+
+	parent_class = g_type_class_peek_parent (g_class);
 }
 
 static void cxp_handler_dispose (GObject * obj)
 {
-	CxpHandler *self = CXP_HANDLER (obj);
+	CxpHandlerPrivate *priv = CXP_HANDLER_GET_PRIVATE(obj);
 
-	if (self->priv->dispose_has_run)
+	if (priv->dispose_has_run)
 	{
 		return;
 	}
 
-	self->priv->dispose_has_run = TRUE;
-	g_object_unref (self->priv->profile);
+	priv->dispose_has_run = TRUE;
+	g_object_unref (priv->profile);
+
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void cxp_handler_finalize (GObject * obj)
 {
-	CxpHandler *self = CXP_HANDLER (obj);
-
-	g_free (self->priv);
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 CxpHandler *cxp_handler_new (void)
@@ -137,6 +144,7 @@ CxpHandler *cxp_handler_new (void)
 
 void cxp_handler_launch (CxpHandler * handler, const gchar * fullpath)
 {
+	CxpHandlerPrivate *priv = CXP_HANDLER_GET_PRIVATE(handler);
 	gchar *filename;
 	gchar *suffix = NULL;
 	gchar *cmd = NULL;
@@ -150,7 +158,7 @@ void cxp_handler_launch (CxpHandler * handler, const gchar * fullpath)
 	filename = g_path_get_basename (fullpath);
 	if ((suffix = g_strrstr (filename, ".")) != NULL)
 	{
-		cmd = cxp_profile_get_string (handler->priv->profile, suffix);
+		cmd = cxp_profile_get_string (priv->profile, suffix);
 		if (cmd == NULL)
 		{
 			message =
@@ -164,11 +172,9 @@ void cxp_handler_launch (CxpHandler * handler, const gchar * fullpath)
 			{
 				cmd = cxp_entry_dialog_get_entry_text
 					(CXP_ENTRY_DIALOG (dialog));
-				if (cmd != NULL)
+				if ((cmd != NULL) && (strlen (cmd) > 0))
 				{
-					cxp_profile_set_string (handler->priv->
-								profile, suffix,
-								cmd);
+					cxp_profile_set_string (priv-> profile, suffix, cmd);
 				}
 			}
 			gtk_widget_destroy (dialog);
@@ -192,7 +198,7 @@ void cxp_handler_launch (CxpHandler * handler, const gchar * fullpath)
 		g_free (message);
 	}
 
-	if (cmd != NULL)
+	if ((cmd != NULL) && (strlen (cmd) > 0))
 	{
 		fullcmd = g_strdup_printf ("%s '%s'", cmd, fullpath);
 		g_spawn_command_line_async (fullcmd, NULL);
