@@ -32,17 +32,26 @@
 #include <sys/types.h>
 #include "cxp-profile.h"
 
+enum
+{
+	CXP_PROFILE_APPNAME = 1,
+	CXP_PROFILE_SECTION,
+};
+
 /**
  * @brief private member of CxpProfile
  */
-
-struct _CxpProfilePrivate
+typedef struct
 {
 	gchar *appname;			/**< application name */
 	gchar *section;			/**< section name */
 	GHashTable *config;		/**< hash table of configuration */
 	gboolean dispose_has_run;	/**< Is dispose funciton executed */
-};
+} CxpProfilePrivate;
+
+#define CXP_PROFILE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CXP_TYPE_PROFILE, CxpProfilePrivate))
+
+static GObjectClass *parent_class = NULL;
 
 /*
  * privete methods
@@ -51,6 +60,8 @@ static void cxp_profile_init (GTypeInstance * instance, gpointer g_class);
 static void cxp_profile_class_init (gpointer g_class, gpointer g_class_data);
 static void cxp_profile_dispose (GObject * obj);
 static void cxp_profile_finalize (GObject * obj);
+static void cxp_profile_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
+static void cxp_profile_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static void cxp_profile_save_file (gpointer key, gpointer value,
 				   gpointer channel);
 static void cxp_profile_load_file (CxpProfile * profile);
@@ -84,19 +95,16 @@ GType cxp_profile_get_type (void)
 }
 
 /**
- * base init CxpProfile
- *
+ * Init instance
  */
 static void cxp_profile_init (GTypeInstance * instance, gpointer g_class)
 {
-	CxpProfile *self = CXP_PROFILE (instance);
+	CxpProfilePrivate *priv = CXP_PROFILE_GET_PRIVATE(instance);
 
-	self->priv = g_new (CxpProfilePrivate, 1);
-	self->priv->appname = NULL;
-	self->priv->section = NULL;
-	self->priv->config =
-		g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-	self->priv->dispose_has_run = FALSE;
+	priv->appname = NULL;
+	priv->section = NULL;
+	priv->config = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	priv->dispose_has_run = FALSE;
 }
 
 /**
@@ -106,46 +114,114 @@ static void cxp_profile_init (GTypeInstance * instance, gpointer g_class)
 static void cxp_profile_class_init (gpointer g_class, gpointer g_class_data)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
+	GParamSpec *pspec;
+
+	g_type_class_add_private (g_class, sizeof (CxpProfilePrivate));
 
 	gobject_class->dispose = cxp_profile_dispose;
 	gobject_class->finalize = cxp_profile_finalize;
+	gobject_class->set_property = cxp_profile_set_property;
+	gobject_class->get_property = cxp_profile_get_property;
+
+	/* Install properties. */
+	pspec = g_param_spec_string ("appname",
+			"Application's name",
+			"Set application's name",
+			NULL,
+			G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+	g_object_class_install_property (gobject_class,
+			CXP_PROFILE_APPNAME,
+			pspec);
+
+	pspec = g_param_spec_string ("section",
+			"Section's name",
+			"Set section's name",
+			NULL,
+			G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+	g_object_class_install_property (gobject_class,
+			CXP_PROFILE_SECTION,
+			pspec);
+
+	parent_class = g_type_class_peek_parent (g_class);
 }
 
 static void cxp_profile_dispose (GObject * obj)
 {
 	CxpProfile *self = CXP_PROFILE (obj);
+	CxpProfilePrivate *priv = CXP_PROFILE_GET_PRIVATE(obj);
 	GIOChannel *channel;
 	gchar *filename;
 
-	if (self->priv->dispose_has_run)
+	if (priv->dispose_has_run)
 	{
 		return;
 	}
 
-	self->priv->dispose_has_run = TRUE;
+	priv->dispose_has_run = TRUE;
 	filename = g_build_filename (g_get_home_dir (),
 				     ".cxp",
-				     self->priv->appname,
-				     self->priv->section, NULL);
+				     priv->appname,
+				     priv->section, NULL);
 	if ((channel = g_io_channel_new_file (filename, "w", NULL)) != NULL)
 	{
-		g_hash_table_foreach (self->priv->config, cxp_profile_save_file,
+		g_hash_table_foreach (priv->config, cxp_profile_save_file,
 				      channel);
 		g_io_channel_flush (channel, NULL);
 
 		g_io_channel_unref (channel);
 	}
 	g_free (filename);
+
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void cxp_profile_finalize (GObject * obj)
 {
-	CxpProfile *self = CXP_PROFILE (obj);
+	CxpProfilePrivate *priv = CXP_PROFILE_GET_PRIVATE(obj);
 
-	g_free (self->priv->appname);
-	g_free (self->priv->section);
-	g_hash_table_destroy (self->priv->config);
-	g_free (self->priv);
+	g_free (priv->appname);
+	g_free (priv->section);
+	g_hash_table_destroy (priv->config);
+
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
+}
+
+static void cxp_profile_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
+{
+	CxpProfilePrivate *private = CXP_PROFILE_GET_PRIVATE(object);
+
+	switch (property_id) {
+		case CXP_PROFILE_APPNAME:
+			g_free (private->appname);
+			private->appname = g_value_dup_string (value);
+			break;
+		case CXP_PROFILE_SECTION:
+			g_free (private->section);
+			private->section = g_value_dup_string (value);
+			break;
+		default:
+			/* We don't have any other property... */
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+			break;
+	}
+}
+
+static void cxp_profile_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
+{
+	CxpProfilePrivate *private = CXP_PROFILE_GET_PRIVATE(object);
+
+	switch (property_id) {
+		case CXP_PROFILE_APPNAME:
+			g_value_set_string (value, private->appname);
+			break;
+		case CXP_PROFILE_SECTION:
+			g_value_set_string (value, private->section);
+			break;
+		default:
+			/* We don't have any other property... */
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+			break;
+	}
 }
 
 CxpProfile *cxp_profile_new (const gchar * appname, const gchar * section)
@@ -153,10 +229,9 @@ CxpProfile *cxp_profile_new (const gchar * appname, const gchar * section)
 	CxpProfile *profile;
 	gchar *cxpdir;
 	gchar *appdir;
+	CxpProfilePrivate *priv;
 
-	profile = CXP_PROFILE (g_object_new (CXP_TYPE_PROFILE, NULL));
-	profile->priv->appname = g_strdup (appname);
-	profile->priv->section = g_strdup (section);
+	profile = CXP_PROFILE (g_object_new (CXP_TYPE_PROFILE, "appname", appname, "section", section, NULL));
 	cxpdir = g_build_filename (g_get_home_dir (), ".cxp", NULL);
 	if (!g_file_test (cxpdir, G_FILE_TEST_EXISTS))
 	{
@@ -175,8 +250,6 @@ CxpProfile *cxp_profile_new (const gchar * appname, const gchar * section)
 	}
 	cxp_profile_load_file (profile);
 
-	g_free (cxpdir);
-	g_free (appdir);
 	return profile;
 }
 
@@ -184,12 +257,13 @@ gint cxp_profile_get_integer (CxpProfile * profile, const gchar * name)
 {
 	gint retval;
 	gchar *value;
+	CxpProfilePrivate *priv = CXP_PROFILE_GET_PRIVATE(profile);
 
-	value = (gchar *) g_hash_table_lookup (profile->priv->config, name);
+	value = (gchar *) g_hash_table_lookup (priv->config, name);
 	if (value == NULL)
 	{
 		retval = 0;
-		g_hash_table_replace (profile->priv->config, g_strdup (name),
+		g_hash_table_replace (priv->config, g_strdup (name),
 				      g_strdup ("0"));
 	}
 	else
@@ -203,8 +277,9 @@ gint cxp_profile_get_integer (CxpProfile * profile, const gchar * name)
 gchar *cxp_profile_get_string (CxpProfile * profile, const gchar * name)
 {
 	gchar *value;
+	CxpProfilePrivate *priv = CXP_PROFILE_GET_PRIVATE(profile);
 
-	value = (gchar *) g_hash_table_lookup (profile->priv->config, name);
+	value = (gchar *) g_hash_table_lookup (priv->config, name);
 
 	return g_strdup (value);
 }
@@ -212,14 +287,18 @@ gchar *cxp_profile_get_string (CxpProfile * profile, const gchar * name)
 void cxp_profile_set_integer (CxpProfile * profile, const gchar * key,
 			      gint value)
 {
-	g_hash_table_replace (profile->priv->config, g_strdup (key),
+	CxpProfilePrivate *priv = CXP_PROFILE_GET_PRIVATE(profile);
+
+	g_hash_table_replace (priv->config, g_strdup (key),
 			      g_strdup_printf ("%d", value));
 }
 
 void cxp_profile_set_string (CxpProfile * profile, const gchar * key,
 			     const gchar * value)
 {
-	g_hash_table_replace (profile->priv->config, g_strdup (key),
+	CxpProfilePrivate *priv = CXP_PROFILE_GET_PRIVATE(profile);
+
+	g_hash_table_replace (priv->config, g_strdup (key),
 			      g_strdup (value));
 }
 
@@ -227,7 +306,8 @@ void cxp_profile_set_string (CxpProfile * profile, const gchar * key,
 static void cxp_profile_load_file (CxpProfile * profile)
 {
 	gchar *filename;
-	GHashTable *config = profile->priv->config;
+	CxpProfilePrivate *priv = CXP_PROFILE_GET_PRIVATE(profile);
+	GHashTable *config = priv->config;
 	gchar *line;
 	gchar **array;
 	gchar *name;
@@ -236,15 +316,15 @@ static void cxp_profile_load_file (CxpProfile * profile)
 
 	filename = g_build_filename (g_get_home_dir (),
 				     ".cxp",
-				     profile->priv->appname,
-				     profile->priv->section, NULL);
+				     priv->appname,
+				     priv->section, NULL);
 	if ((channel = g_io_channel_new_file (filename, "r", NULL)) == NULL)
 	{
 		g_free (filename);
 		filename =
 			g_build_filename (PACKAGE_CONF_DIR,
-					  profile->priv->appname,
-					  profile->priv->section, NULL);
+					  priv->appname,
+					  priv->section, NULL);
 		channel = g_io_channel_new_file (filename, "r", NULL);
 	}
 
